#!/usr/bin/env python3

"""Script which waits for peers to join an ES cluster."""

import logging
import os
import socket
import sys
import time
import traceback

logging.basicConfig(
    stream=sys.stderr,
    level=logging.DEBUG,
    format="%(asctime)s [%(levelname)s] %(message)s",
)

import requests  # noqa: E402

sys.path.append(os.path.abspath("mod/charm-helpers"))
from charmhelpers.core.host import service_restart  # noqa: I100,E402


CONNECTION_TIMEOUT = 5
POLL_DELAY = 5
SCRIPT_TIMEOUT = 600


def main():
    """Run the script."""
    start_time = time.time()
    while True:
        node_count = get_node_count()
        if node_count > 1:
            logging.info("Multiple ES nodes detected; success.")
            return 0
        if time.time() - start_time > SCRIPT_TIMEOUT:
            logging.error("Timed out while waiting for peers")
            return 1
        logging.info("Node count is not greater than 1, will restart and retry")
        logging.info("Restarting elasticsearch...")
        restart_elasticsearch()
        logging.info("Waiting for elasticsearch port to open...")
        wait_for_es_port()
        logging.info("Waiting {} seconds...".format(POLL_DELAY))
        time.sleep(POLL_DELAY)


def get_node_count():
    """Return the number of nodes detected in the cluster.

    ES communication errors are assumed to be temporary, and in such cases this function
    returns 0.

    """
    # NOTE: 503 errors have been intermittently sighted here, so we'll retry
    # in case of errors.
    cluster_health_url = "http://localhost:9200/_cluster/health"
    max_attempts = 5
    for i in range(max_attempts):
        try:
            return requests.get(cluster_health_url, timeout=CONNECTION_TIMEOUT).json()[
                "number_of_nodes"
            ]
        except Exception:
            if i < max_attempts - 1:
                logging.info(
                    "Error occurred while polling {}; retrying".format(
                        cluster_health_url
                    )
                )
                time.sleep(POLL_DELAY)
            else:
                logging.warning(traceback.format_exc())
    else:
        logging.warning(
            "Error occurred while polling {}; giving up".format(cluster_health_url)
        )
        return 0


def restart_elasticsearch():
    """Restart the ElasticSearch service."""
    service_restart("elasticsearch")


def wait_for_es_port(timeout=60):
    """Wait for the ElasticSearch port to open."""
    start_time = time.time()
    while time.time() - start_time < timeout:
        try:
            _test_socket_connection("localhost", 9200)
        except Exception:
            time.sleep(POLL_DELAY)
        else:
            return
    else:
        # Last try; allow exceptions to bubble up
        _test_socket_connection("localhost", 9200)


def _test_socket_connection(host, port):
    s = socket.create_connection((host, port), timeout=CONNECTION_TIMEOUT)
    s.close()


if __name__ == "__main__":
    sys.exit(main())
