#!/usr/bin/python3
"""Tests which scale ES from one to multiple nodes."""

import json
import math
import time
import unittest

import tenacity

from zaza import model

CLUSTER_NAME = "unique-name"
CURL_TIMEOUT = 180
JUJU_TIMEOUT = 1200


class TestElasticsearch(unittest.TestCase):
    """Tests which scale ES from one to multiple nodes."""

    def test_01_health(self):
        """Single node health checks.

        Test the health of the node upon first deployment by getting the cluster health,
        then inserting data and validating cluster health

        """
        health = self.get_cluster_health()
        self.assertIn(health["status"], ("green", "yellow"))

        # Create a test index.
        curl_command = """
        curl -XPUT 'http://localhost:9200/test/tweet/1' -d '{
            "user" : "me",
            "message" : "testing"
        }'
        """
        self.curl_on_unit(curl_command)
        health = self.get_index_health("test")
        self.assertIn(health["status"], ("green", "yellow"))

    def test_02_config(self):
        """Verify cluster_name configuration."""
        health = self.get_cluster_health()
        cluster_name = health["cluster_name"]
        self.assertEqual(cluster_name, CLUSTER_NAME)

    def test_03_scale(self):
        """Test cluster scaling.

        Validate scaling the elasticsearch cluster yields a healthy response from the
        API, and all units are participating.

        """
        model.add_unit("elasticsearch", count=2)
        for i in range(3):
            self._block_until("elasticsearch/{}".format(i), "active", "Ready")

        # On slower machines, this may not be ready right away.
        for attempt in tenacity.Retrying(
            wait=tenacity.wait_fixed(10),
            stop=tenacity.stop_after_attempt(10),
            reraise=True,
        ):
            with attempt:
                health = self.get_cluster_health(wait_for_nodes=3)
                index_health = self.get_index_health("test")
                print(health["number_of_nodes"])
                self.assertGreaterEqual(health["number_of_nodes"], 3)
                self.assertIn(index_health["status"], ("green", "yellow"))

    def curl_on_unit(self, curl_command):
        """Run a curl command on the sentried ES unit.

        We'll retry this command until the CURL_TIMEOUT because it may take a few
        seconds for the ES systemd service to start.

        """
        timeout = time.time() + CURL_TIMEOUT
        while time.time() < timeout:
            results = model.run_on_unit(
                "elasticsearch/0", curl_command, timeout=timeout
            )
            if results["Code"] == "0":
                return json.loads(results["Stdout"])
            else:
                print(
                    "Unexpected curl response: {}. "
                    "Retrying in 30s.".format(results["Stderr"])
                )
                time.sleep(30)

        msg = (
            "Elastic search didn't respond to the command \n"
            "'{curl_command}' as expected.\n"
            "Return code: {return_code}\n"
            "Result: {result}".format(
                curl_command=curl_command,
                return_code=results["Code"],
                result=results["Stderr"],
            )
        )
        self.fail(msg)

    def get_cluster_health(self, wait_for_nodes=0):
        """Query the current cluster health of the ES cluster."""
        curl_url = "http://localhost:9200/_cluster/health"
        if wait_for_nodes > 0:
            # Give the API up to 3m to determine if desired nodes are present
            curl_url = curl_url + "?timeout=180s&wait_for_nodes=ge({})".format(
                wait_for_nodes
            )
        curl_command = "curl -XGET '{}'".format(curl_url)

        return self.curl_on_unit(curl_command)

    def get_index_health(self, index_name):
        """Query the current index health of the ES cluster."""
        curl_url = "http://localhost:9200/_cluster/health/" + index_name
        curl_command = "curl -XGET '{}'".format(curl_url)

        return self.curl_on_unit(curl_command)

    def _block_until(
        self,
        unit,
        expected_workload_status,
        expected_workload_status_message,
        timeout=1800,
    ):
        start_time = time.time()

        def get_remaining_timeout():
            return max(0, math.ceil((start_time + timeout) - time.time()))

        # Zaza doesn't presently seem to provide a way to check these 3 things
        # at the same time.  I could write a helper to do this, but this may be "good
        # enough" for the time being.
        app = unit.split("/")[0]
        model.block_until_unit_wl_status(
            unit, expected_workload_status, timeout=timeout
        )
        model.block_until_wl_status_info_starts_with(
            app, expected_workload_status_message, timeout=get_remaining_timeout()
        )
        model.wait_for_unit_idle(unit, timeout=get_remaining_timeout())


if __name__ == "__main__":
    unittest.main()
